#! /bin/sh
# Prepare files needed for flashing a Colbiri VFxx module
# and copy them into a convienent location for use from a running u-boot

# sometimes we need the binary echo, not the shell builtin
ECHO=`which echo`

Flash()
{
	echo "To flash the Colibri Vybrid module, boot the module to the u-boot prompt and"
	echo ""
	echo "when using a SD-card, connect the SD-card and enter:"
	echo "mmc part 0 ; fatload mmc 0:1 \$loadaddr flash_mmc.img ;source"
	echo ""
	echo "when using tftp, connect Ethernet and enter:"
	echo "tftp \$loadaddr flash_eth.img ; source"
	echo ""
	echo "to update all enter:"
	echo "run update"
	echo ""
	echo "to update a single component enter one of:"
	echo "run update_uboot"
	echo "run update_kernel"
	echo "run update_rootfs"
	echo ""
	echo ""
	echo "If you don't have a working u-boot anymore, connect your PC to the module's"
	echo "UART, bring the module in the serial download mode and start"
	echo "the update.sh script with the -d option. This will copy u-boot into the"
	echo "modules RAM and execute it."
	echo "./update.sh -d /dev/ttyUSB0"
}

Usage()
{
	echo ""
	echo "Prepares and copies files for flashing the internal NAND of a Colibri VFxx"
	echo ""
	echo "Will require a running u-boot on the target. Either one already flashed"
	echo "on the NAND or strapping the board to boot from SD-card and booting"
	echo "u-boot from the card."
	echo "The script format_sd.sh may be used to format a SD-card"
	echo ""
	echo "-d uart dev  : use a UART connection to copy and execute u-boot to the module's RAM"
	echo "-h           : Prints this message"
	echo "-n           : Disable hardware flow control (bridge RTS/CTS!)"
	echo "-o directory : output directory, defaults to /media/KERNEL"
	echo "-s           : optimize filesystem for 128MB NAND, increases usable space"
	echo "               on VF50 module a little, but on VF61 uses also only 128MB"
	echo ""
	echo "Example \"./update.sh\" copies the requiered files to /media/KERNEL/"
	echo "Example \"./update.sh -o /srv/tftp/\" copies the requiered files to /srv/tftp/"
	echo ""
	echo ""
	Flash
}

#initialise options
UBOOT_RECOVERY=0
NORTSCTS=0
OUT_DIR="/media/KERNEL"
# NAND parameters
PAGE=2KiB
BLOCK=124KiB
MAXLEB=8112

while getopts "d:nho:s" Option ; do
	case $Option in
		d)	UBOOT_RECOVERY=1
			UARTDEV=$OPTARG
			;;
		n)  NORTSCTS=1
			;;
		h) 	Usage
			# Exit if only usage (-h) was specfied.
			if [[ $# -eq 1 ]] ; then
				exit 10
			fi
			exit 0
			;;
		o)	OUT_DIR=$OPTARG
			;;
		s)	MAXLEB=982
			;;
	esac
done

# autotect MODTYPE from from rootfs directory
CNT=`grep -c "VF" rootfs/etc/issue || true`
if [ "$CNT" -ge 1 ] ; then
	echo "Colibri VF rootfs detected"
	MODTYPE=colibri-vf
	IMAGEFILE=ubifs.img

	LOCPATH="vf_flash"
else
	echo "can not detect modulue type from ./rootfs/etc/issue"
	echo "exiting"
	exit 1
fi
BINARIES=${MODTYPE}_bin

#is only u-boot to be copied to RAM?
if [ "$UBOOT_RECOVERY" -ge 1 ] ; then
	LOADEROPTS=""
	if [ ${NORTSCTS} == 1 ]; then
		LOADEROPTS="--no-rtscts"
	fi

	read -p "Put the module in recovery mode and press [ENTER]..."
	sudo ${LOCPATH}/imx_uart ${LOADEROPTS} ${UARTDEV} ${LOCPATH}/vybrid_usb_work.conf ${BINARIES}/u-boot.nand
	# Stop U-Boot auto boot
	sleep 1 && echo "" > ${UARTDEV}
	exit 1
fi

# is OUT_DIR an existing directory?
if [ ! -d "$OUT_DIR" ] ; then
	echo "$OUT_DIR" "does not exist, exiting"
	exit 1
fi

#sanity check for correct untared rootfs
DEV_OWNER=`ls -ld rootfs/dev | awk '{print $3}'`
if [ "${DEV_OWNER}x" != "rootx" ]
then
	$ECHO -e "rootfs/dev is not owned by root, but it should!"
	$ECHO -e "\033[1mPlease unpack the tarball with root rights.\033[0m"
	$ECHO -e "e.g. sudo tar xjvf Colibri_VF50_LinuxImageV2.1_20130924.tar.bz2"
	exit 1
fi

#sanity check, can we execute mkfs.ubifs, e.g. see the help text?
CNT=`sudo $LOCPATH/mkfs.ubifs -h | grep -c space-fixup`  
if [ "$CNT" -eq 0 ] ; then
	echo "The program mkfs.ubifs can not be executed or does not provide --space-fixup option"
	echo "Are you on a 64bit Linux host without installed 32bit execution environment?"
	$ECHO -e  "\033[1mPlease install e.g. ia32-libs on 64-bit Ubuntu\033[0m"
	$ECHO -e  "\033[1mMaybe others are needed e.g. liblzo2:i386 on 64-bit Ubuntu\033[0m"
	exit 1
fi

# Prepare full flashing
#build ${IMAGEFILE} if it does not exist
sudo $LOCPATH/mkfs.ubifs -c ${MAXLEB} -e ${BLOCK} -m ${PAGE} -o ${BINARIES}/${IMAGEFILE} -r rootfs/ -v

#make a file with the used versions for u-boot, kernel and rootfs
sudo touch ${BINARIES}/versions.txt
sudo chmod ugo+w ${BINARIES}/versions.txt
echo "Component Versions" > ${BINARIES}/versions.txt
basename "`readlink -e ${BINARIES}/u-boot.nand`" >> ${BINARIES}/versions.txt
basename "`readlink -e ${BINARIES}/uImage`" >> ${BINARIES}/versions.txt
$ECHO -n "Rootfs " >> ${BINARIES}/versions.txt
grep VF rootfs/etc/issue >> ${BINARIES}/versions.txt

#copy to $OUT_DIR
sudo cp ${BINARIES}/configblock.bin ${BINARIES}/u-boot.nand ${BINARIES}/uImage ${BINARIES}/ubifs.img ${BINARIES}/flash*.img ${BINARIES}/versions.txt "$OUT_DIR"
sync

Flash
